import os
import sys
import argparse
import importlib

from espresso.configuration import Configuration
from espresso.stage import StageType


def setup():
    """ 
    This function must create the minimum os needs to use 
    espresso. It needs root access.

    We basically create a directory at /etc/espresso and create there
    some others like conf.d and more.
    """

    # Check that we have root access.
    if not os.geteuid() == 0:
        sys.exit("This script must be run by root.")

    # Create main path for configurations.
    os.mkdir('/etc/espresso')
    os.mkdir('/etc/espresso/conf.d')


def main(argv=None):
    """
    Basic cli for espresso.

    For now we just need to handle configurations.

    """

    # Create parser.
    parser = argparse.ArgumentParser(description="Espresso command-line interface")

    # # Select the namespace used.
    # parser.add_argument('--namespace', help="Select namespace for any operation.", 
    #                     required=True)
    
    # # Add argument.
    # parser.add_argument('--add-option', nargs=2, dest="add_option", default=None,
    #                     help="Add a new option, eg: general.ip 1.1.1.1")

    # Load stages.
    parser.add_argument(
        '-S', '--stages', dest='stages', 
        help="Importable module with the stages", required=True)
    
    parser.add_argument('-R', '--run', action='append', help="Run this stage")

    # Parse the args.
    args = parser.parse_args(argv)

    # # Check if we want to add an option.
    # if args.add_option is not None:

    #     # Get section and name.
    #     section, name = args.add_option[0].split('.')

    #     # Get value.
    #     value = args.add_option[1]

    #     # Use the namespace.
    #     Configuration(args.namespace).set(section, name, value)

    # We need to load some stages.
    stages_module = importlib.import_module(args.stages)

    # Real stages.
    stages = {}
    for name in stages_module.__dict__:

        # Check if its a stage.
        if not isinstance(stages_module.__dict__[name], StageType):
            continue

        # Add stage by name
        stage = stages_module.__dict__[name]

        # We can't do anything without a name.
        if 'name' not in stage._meta:
            continue

        stages[stage._meta['name']] = stage

    # Check that all the stages exists.
    for stage in args.run:
        if stage not in stages:
            raise SystemError("Non existing stage: '%s'" % (stage, ))

    # Run stages.
    for stage in args.run:
        stages[stage]().run()
        



import os

class Storage(object):
    """
    Storage
    =======

    This defines an abstract storage it can be a local storage, http, and so on.

    It works basically as a dict with __getitem__.

    """

    def __getitem__(self, name):
        raise ValueError("Storage's backend is not implemented yet.")


class DirectoryStorage(object):
    """
    Directory's storage
    ===================

    Lock for files in a list of directories in order.

    Example::

        from espresso.storage import DirectoryStorage
        import os
        
        storage = DirectoryStorage(['/tmp', '/home/user', os.path.dirname(__file__)])

        print storage['bridge.template']


    """

    def __init__(self, directories=[]):
        self.directories = directories

    def __getitem__(self, name):
        " Look for object."

        for directory in self.directories:
            if os.path.isdir(directory):
                if name in os.listdir(directory):
                    with open(os.path.join(directory, name)) as fd:
                        content = fd.read()
                    return content

        raise ValueError("The object doesn't exists in the storage.")



from setuptools import setup, find_packages

setup(
    name='espresso',
    description='Manage the configuration of your servers easily.',
    version='1.0.14',
    url='http://github.com/jorgeecardona/espresso',
    author='Jorge E. Cardona',
    author_email='jorge@cardona.co',
    packages=['espresso'],
    license="BSD",
    test_suite='tests',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: BSD License",
        ],
    entry_points={
        'console_scripts': [
            'setup-espresso = espresso.cli:setup',
            'espresso = espresso.cli:main'
            ],
        },
    install_requires=[
        'configparser>=3.2', 
        'argparse',
        'distribute',
        'fstab',
        'sh'],
    setup_requires=['distribute']
    )
